#ifndef BASECALC_H
#define BASECALC_H

#include <QThread>
#include <QVector>
#include <QPoint>
#include <QLine>
#include <QColor>
#include "math.h"

bool operator<(QPoint a, QPoint b);
//bool operator<(QLine a, QLine b);

class BaseCalc : public QThread
{
    Q_OBJECT
public:
    static const int maxBenween2 = 100;
    static const int maxPst = 100;
    static const int maxW=70;
    static const int maxH=70;

    explicit BaseCalc(QObject *parent = 0);

    QVector<int> makeMatrix(int n);
    QString printMatrix(QVector<int>& matr);

    virtual void calculate(QVector<int>& matr);
    virtual QString getName() = 0;

    int getPath() { return minPath; }
    QVector<int> getPoints() { return minPathPoints; }

    QVector<QPoint> getTowns() { return towns; }

signals:
    void progress(int p); // 0..100
    void drawLine(QPoint fromTo, QColor color);

public slots:
    void drawPath(QVector<int> been, QColor color);

protected:
    int count;
    int minPath;
    QVector<int> minPathPoints;
    qint64 iters;
    qint64 current;
    QVector<QPoint> towns;
    QVector<int> matrix; // mCount*mCount elements

    qint64 fact(int i);
    void recursive(QVector<int> been, QVector<int> will);
};

#endif // BASECALC_H
