#include "graf.h"

bool operator<(QPoint a, QPoint b)
{
    if (a.x() < b.x()) return true;
    if (a.x() > b.x()) return false;

    //if (a.y() < b.y()) return true;
    //if (a.y() > b.y()) return false;

    return false;
}

Graf::Graf(QWidget *parent) :
    QWidget(parent)
{
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    timer->start(100);
}

Graf::~Graf(){
    //delete data;
}

void Graf::paintEvent(QPaintEvent *)
{
    clock();

    QColor townColor(127, 127, 0, 191);

    int side = qMin(width(), height());

    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    if (width()>height())
        painter.translate((width()-height()) / 2, 0);
    else
        painter.translate(0, -(width()-height()) / 2);
    painter.scale(side / 70.0, side / 70.0);

    painter.setPen(Qt::NoPen);
    painter.setBrush(townColor);

    //painter.save();
    //painter.rotate(30.0 * ((time.hour() + time.minute() / 60.0)));
    //painter.restore();

    foreach (QPoint town, towns){
        painter.drawEllipse(town, 2, 2);
    }
    QPen pen;
    pen.setWidth(1);
    QMap<QPoint, QColor>::iterator pair;
    for (pair=lines.begin(); pair!=lines.end(); ++pair){
        int a = pair.key().x();
        int b = pair.key().y();
        QPoint A, B;
        int n=towns.count();
        if (a<n) A = towns[a]; else continue;
        if (b<n) B = towns[b]; else continue;
        pen.setColor(pair.value());
        painter.setPen(pen);
        painter.drawLine(A, B);
    }
}

void Graf::drawLine(QPoint fromTo, QColor color){
    //if (lines[fromTo] == QColor())
    {
        lines[fromTo] = color;
        return;
    }
    int r = (color.red() + lines[fromTo].red())/2;
    int g = (color.green() + lines[fromTo].green())/2;
    int b = (color.blue() + lines[fromTo].blue())/2;
    int a = (color.alpha() + lines[fromTo].alpha())/2;
    lines[fromTo] = QColor(r,g,b,a);
}

void Graf::clock()
{
    static const QPoint hourHand[3] = {
        QPoint(7, 8),
        QPoint(-7, 8),
        QPoint(0, -40)
    };
    static const QPoint minuteHand[3] = {
        QPoint(7, 8),
        QPoint(-7, 8),
        QPoint(0, -70)
    };
    static const QPoint secundHand[3] = {
        QPoint(5, 8),
        QPoint(-5, 8),
        QPoint(0, -75)
    };

    QColor hourColor(127, 0, 127);
    QColor minuteColor(0, 127, 127, 191);
    QColor secundColor(127, 127, 0, 191);

    int side = qMin(width(), height());
    QTime time = QTime::currentTime();

    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.translate(width() / 2, height() / 2);
    painter.scale(side / 200.0, side / 200.0);

    painter.setPen(Qt::NoPen);
    painter.setBrush(hourColor);

    painter.save();
    painter.rotate(30.0 * ((time.hour() + time.minute() / 60.0)));
    painter.drawConvexPolygon(hourHand, 3);
    painter.restore();

    painter.setPen(hourColor);

    for (int i = 0; i < 12; ++i) {
        painter.drawLine(88, 0, 96, 0);
        painter.rotate(30.0);
    }

    painter.setPen(Qt::NoPen);
    painter.setBrush(minuteColor);

    painter.save();
    painter.rotate(6.0 * (time.minute() + time.second() / 60.0));
    painter.drawConvexPolygon(minuteHand, 3);
    painter.restore();

    painter.setPen(minuteColor);

    for (int j = 0; j < 60; ++j) {
        if ((j % 5) != 0)
            painter.drawLine(92, 0, 96, 0);
        painter.rotate(6.0);
    }

    painter.setPen(Qt::NoPen);
    painter.setBrush(secundColor);

    painter.save();
    painter.rotate(6.0 * time.second());
    painter.drawConvexPolygon(secundHand, 3);
    painter.restore();

    painter.setPen(secundColor);

    for (int j = 0; j < 60; ++j) {
        if ((j % 5) != 0)
            painter.drawLine(92, 0, 96, 0);
        painter.rotate(6.0);
    }
}
