#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "basecalc.h"

#include "brutforse.h"
#include "comymin.h"
#include "comymrand.h"
#include "comyrand.h"
#include "comywrand.h"
#include "comyant.h"
#include "graf.h"

#include <QDateTime>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    mCount(10),
    printMatrix(false),
    step(0),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    QTimer *timer = new QTimer(this);

    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    timer->start(1000);

    setWindowTitle(QString::fromUtf8("Коммивояжёр"));
    resize(800, 500);

    connect(ui->pushButton, SIGNAL(clicked()), this, SLOT(calculate()));

    QTextCharFormat fmt = ui->textEdit->currentCharFormat();
    fmt.setFontFixedPitch(true);
    fmt.setFontStyleHint(QFont::Monospace);
    fmt.setFontFamily("monotype");
    ui->textEdit->setCurrentCharFormat(fmt);

    connect(ui->spinBox, SIGNAL(valueChanged(int)), this, SLOT(countChanged(int)));
    mCount = ui->spinBox->value();

    //connect(ui->radioButton_brutForce, SIGNAL(toggled(bool)), this, SLOT(brut14(bool)));

    graf = new Graf(this);
    ui->tab_graf->layout()->addWidget(graf);

    qRegisterMetaType<QVector<int> >("QVector<int>");
    countChanged(mCount);

    qsrand(QDateTime::currentMSecsSinceEpoch());
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::paintEvent(QPaintEvent *)
{
}

void MainWindow::progress(int p)
{
    ui->progressBar->setValue(p);
}

void MainWindow::countChanged(int co){
    mCount=co;
    BrutForse bk;
    matr = bk.makeMatrix(mCount);
    graf->clearLines();
    graf->townsInit(bk.getTowns());
    if (mCount > 14) {
        ui->radioButton_brutForce->setEnabled(false);
        ui->textEdit->append(tr("Matrix %1 x %1").arg(co));
        if (ui->radioButton_brutForce->isChecked())
            ui->radioButton_min->setChecked(true);
            //ui->radioButton_brutForce->setChecked(false);
    } else {
        ui->radioButton_brutForce->setEnabled(true);
        QString s = bk.printMatrix(matr);
        ui->textEdit->append(s);
    }
}

void MainWindow::finished()
{
    progress(BaseCalc::maxPst);
    int ms1 = QDateTime::currentMSecsSinceEpoch() - dt;
    int rezBF = calcMethods[step]->getPath();
    QVector<int> rezPoints = calcMethods[step]->getPoints();

    ui->textEdit->append(QString::fromUtf8("решение '%1'\n").
                arg(calcMethods[step]->getName())+
                tr("minPath = %1, time = %2 msec").arg(rezBF).arg(ms1));
    QString points;
    for (int i=0; i<rezPoints.count(); ++i)
        points += tr("%1, ").arg(rezPoints[i]);
    points.remove(points.count()-2, 2);
    ui->textEdit->append(tr("(")+points+")\n");

    disconnect(this, SLOT(progress(int)));
    disconnect(this, SLOT(finished()));
    disconnect(graf, SLOT(drawLine(QPoint,QColor)));

    step++;
    if (step >= calcMethods.count()){
        ui->pushButton->setEnabled(true);
        //ui->textEdit->append(QString::fromUtf8("Готово!\n"));
    } else {
        connect(calcMethods[step], SIGNAL(progress(int)), this, SLOT(progress(int)));
        connect(calcMethods[step], SIGNAL(finished()), this, SLOT(finished()));
        connect(calcMethods[step], SIGNAL(drawLine(QPoint,QColor)),
                graf, SLOT(drawLine(QPoint,QColor)));
        dt = QDateTime::currentMSecsSinceEpoch();
        calcMethods[step]->start();
    }
}

void MainWindow::calculate()
{
    if (step < calcMethods.count()) return;

    //qsrand(QDateTime::currentDateTime().msecsTo(QDateTime())); //currentMSecsSinceEpoch());
    qsrand(QTime(0,0,0).msecsTo(QTime::currentTime()));
    ui->pushButton->setEnabled(false);
    while (calcMethods.count() > 0){
        BaseCalc* bk = calcMethods[0];
        //if (bk->isFinished())
        {
            delete bk;
            calcMethods.remove(0);
        }
    }

    //BaseCalc bk(this);
    BrutForse bk;
    graf->clearLines();
    if (printMatrix){
        QString s = bk.printMatrix(matr);
        ui->textEdit->append(s);
    }

    if (ui->radioButton_brutForce->isChecked()){
        BrutForse* bf = new BrutForse;
        calcMethods << bf;
        bf->calculate(matr);
    }

    if (ui->radioButton_min->isChecked()){
        ComyMin* cm = new ComyMin;
        calcMethods << cm;
        cm->calculate(matr);
    }

    if (ui->radioButton_rand->isChecked()){
        //ComyRand* cr = new ComyRand;
        //calcMethods << cr;
        //cr->calculate(matr);
        //ComyWrand* cwr = new ComyWrand;
        //calcMethods << cwr;
        //cwr->calculate(matr);
        ComyMrand* mr = new ComyMrand;
        calcMethods << mr;
        mr->calculate(matr);
    }

    if (ui->radioButton_ant->isChecked()){
        ComyAnt* ca = new ComyAnt;
        calcMethods << ca;
        ca->calculate(matr);
    }

    if (ui->radioButton_bee->isChecked()){
        //
    }

    if (calcMethods.count()>0){
        step = 0;
        connect(calcMethods[step], SIGNAL(progress(int)), this, SLOT(progress(int)));
        connect(calcMethods[step], SIGNAL(finished()), this, SLOT(finished()));
        connect(calcMethods[step], SIGNAL(drawLine(QPoint,QColor)),
                graf, SLOT(drawLine(QPoint,QColor)));
        dt = QDateTime::currentMSecsSinceEpoch();
        calcMethods[0]->start();
    }
}

void MainWindow::brut14(bool checked)
{
    static int last;
    if (checked){
        last = ui->spinBox->value();
        if (last > 14){
            ui->spinBox->setMaximum(14);
            ui->spinBox->setValue(9);
        }
        printMatrix = true;
    } else {
        ui->spinBox->setMaximum(100);
        if (last>14)
            ui->spinBox->setValue(last);
        printMatrix = false;
    }
}
